summaryrefslogtreecommitdiff
path: root/mpfr/set_ui.c
diff options
context:
space:
mode:
Diffstat (limited to 'mpfr/set_ui.c')
-rw-r--r--mpfr/set_ui.c85
1 files changed, 48 insertions, 37 deletions
diff --git a/mpfr/set_ui.c b/mpfr/set_ui.c
index 9fd88dd98..2ae08b7ff 100644
--- a/mpfr/set_ui.c
+++ b/mpfr/set_ui.c
@@ -5,64 +5,75 @@ Copyright (C) 1999-2001 Free Software Foundation.
This file is part of the MPFR Library.
The MPFR Library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Library General Public License as published by
-the Free Software Foundation; either version 2 of the License, or (at your
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.
The MPFR Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
License for more details.
-You should have received a copy of the GNU Library General Public License
+You should have received a copy of the GNU Lesser General Public License
along with the MPFR Library; see the file COPYING.LIB. If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. */
-#include <stdio.h>
#include "gmp.h"
#include "gmp-impl.h"
#include "longlong.h"
#include "mpfr.h"
#include "mpfr-impl.h"
-void
-#if __STDC__
-mpfr_set_ui (mpfr_ptr x, unsigned long int i, mp_rnd_t rnd_mode)
-#else
-mpfr_set_ui (x, i, rnd_mode)
- mpfr_ptr x;
- unsigned long int i;
- mp_rnd_t rnd_mode;
-#endif
+int
+mpfr_set_ui (mpfr_ptr x, unsigned long i, mp_rnd_t rnd_mode)
{
- unsigned int xn, cnt; mp_limb_t *xp;
+ int inex = 0;
MPFR_CLEAR_FLAGS(x);
- if (i==0) { MPFR_SET_ZERO(x); return; }
- xn = (MPFR_PREC(x)-1)/BITS_PER_MP_LIMB;
- count_leading_zeros(cnt, (mp_limb_t) i);
-
- xp = MPFR_MANT(x);
- xp[xn] = ((mp_limb_t) i) << cnt;
- /* don't forget to put zero in lower limbs */
- MPN_ZERO(xp, xn);
-
- MPFR_EXP(x) = BITS_PER_MP_LIMB - cnt;
-
- /* round if MPFR_PREC(x) smaller than length of i */
- if (MPFR_PREC(x) < BITS_PER_MP_LIMB-cnt) {
- cnt = mpfr_round_raw(xp+xn, xp+xn, BITS_PER_MP_LIMB-cnt, 0, MPFR_PREC(x),
- rnd_mode);
- if (cnt) { /* special case 1.000...000 */
- MPFR_EXP(x)++;
- xp[xn] = ((mp_limb_t) 1) << (BITS_PER_MP_LIMB-1);
+ if (i == 0)
+ MPFR_SET_ZERO(x); /* the sign will be set later */
+ else
+ {
+ mp_size_t xn;
+ unsigned int cnt, nbits;
+ mp_limb_t *xp;
+
+ xn = (MPFR_PREC(x)-1)/BITS_PER_MP_LIMB;
+ count_leading_zeros(cnt, (mp_limb_t) i);
+
+ xp = MPFR_MANT(x);
+ xp[xn] = ((mp_limb_t) i) << cnt;
+ /* don't forget to put zero in lower limbs */
+ MPN_ZERO(xp, xn);
+
+ MPFR_EXP(x) = nbits = BITS_PER_MP_LIMB - cnt;
+ inex = mpfr_check_range(x, rnd_mode);
+ if (inex)
+ return inex; /* underflow or overflow */
+
+ /* round if MPFR_PREC(x) smaller than length of i */
+ if (MPFR_PREC(x) < nbits)
+ {
+ int carry;
+ carry = mpfr_round_raw(xp+xn, xp+xn, nbits, 0, MPFR_PREC(x),
+ rnd_mode, &inex);
+ if (carry)
+ {
+ mp_exp_t exp = MPFR_EXP(x);
+
+ if (exp == __mpfr_emax)
+ return mpfr_set_overflow(x, rnd_mode, 1);
+
+ MPFR_EXP(x)++;
+ xp[xn] = MP_LIMB_T_HIGHBIT;
+ }
+ }
}
- }
/* warning: don't change the precision of x! */
- if (MPFR_SIGN(x) < 0) MPFR_CHANGE_SIGN(x);
+ if (MPFR_SIGN(x) < 0)
+ MPFR_CHANGE_SIGN(x);
- return;
+ MPFR_RET(inex);
}
-